@@ -43,10 +43,12 @@ struct DesktopEntry
 
   GQuark *categories;
 
-  char *name;
-  char *comment;
-  char *icon;
-  char *exec;
+  char     *name;
+  char     *generic_name;
+  char     *comment;
+  char     *icon;
+  char     *exec;
+  gboolean terminal;
 
   guint type : 2;
   guint flags : 4;
@@ -204,6 +206,7 @@ desktop_entry_load (DesktopEntry *entry)
   GKeyFile     *key_file;
   GError       *error;
   const char   *desktop_entry_group;
+  char         *name_str;
   char         *type_str;
 
   key_file = g_key_file_new ();
@@ -244,6 +247,15 @@ desktop_entry_load (DesktopEntry *entry)
       goto out;
     }
 
+  name_str = g_key_file_get_locale_string (key_file, desktop_entry_group, "Name", NULL, NULL);
+  if (!name_str)
+    {
+      menu_verbose ("\"%s\" contains an invalid \"Name\" key\n", entry->path);
+      goto out;
+    }
+
+  g_free (name_str);
+
   type_str = g_key_file_get_string (key_file, desktop_entry_group, "Type", NULL);
   if (!type_str)
     {
@@ -255,9 +267,12 @@ desktop_entry_load (DesktopEntry *entry)
       (entry->type == DESKTOP_ENTRY_DIRECTORY && strcmp (type_str, "Directory") != 0))
     {
       menu_verbose ("\"%s\" does not contain the correct \"Type\" value\n", entry->path);
+      g_free (type_str);
       goto out;
     }
 
+  g_free (type_str);
+
   if (entry->type == DESKTOP_ENTRY_DESKTOP &&
       !g_key_file_has_key (key_file, desktop_entry_group, "Exec", NULL))
     {
@@ -265,26 +280,29 @@ desktop_entry_load (DesktopEntry *entry)
       goto out;
     }
 
-  g_free (type_str);
-
   retval = entry;
 
 #define GET_LOCALE_STRING(n) g_key_file_get_locale_string (key_file, desktop_entry_group, (n), NULL, NULL)
 
-  retval->name       = GET_LOCALE_STRING ("Name");
-  retval->comment    = GET_LOCALE_STRING ("Comment");
-  retval->icon       = GET_LOCALE_STRING ("Icon");
-  retval->flags      = get_flags_from_key_file (retval, key_file, desktop_entry_group);
-  retval->categories = get_categories_from_key_file (retval, key_file, desktop_entry_group);
+  retval->name         = GET_LOCALE_STRING ("Name");
+  retval->generic_name = GET_LOCALE_STRING ("GenericName");
+  retval->comment      = GET_LOCALE_STRING ("Comment");
+  retval->icon         = GET_LOCALE_STRING ("Icon");
+  retval->flags        = get_flags_from_key_file (retval, key_file, desktop_entry_group);
+  retval->categories   = get_categories_from_key_file (retval, key_file, desktop_entry_group);
 
   if (entry->type == DESKTOP_ENTRY_DESKTOP)
-    retval->exec = g_key_file_get_string (key_file, desktop_entry_group, "Exec", NULL);
+    {
+      retval->exec = g_key_file_get_string (key_file, desktop_entry_group, "Exec", NULL);
+      retval->terminal = g_key_file_get_boolean (key_file, desktop_entry_group, "Terminal", NULL);
+    }
   
 #undef GET_LOCALE_STRING
 
-  menu_verbose ("Desktop entry \"%s\" (%s, %s, %s) flags: NoDisplay=%s, Hidden=%s, ShowInGNOME=%s, TryExecFailed=%s\n",
+  menu_verbose ("Desktop entry \"%s\" (%s, %s, %s, %s) flags: NoDisplay=%s, Hidden=%s, ShowInGNOME=%s, TryExecFailed=%s\n",
                 retval->basename,
                 retval->name,
+                retval->generic_name ? retval->generic_name : "(null)",
                 retval->comment ? retval->comment : "(null)",
                 retval->icon ? retval->icon : "(null)",
                 retval->flags & DESKTOP_ENTRY_NO_DISPLAY     ? "(true)" : "(false)",
@@ -347,6 +365,9 @@ desktop_entry_reload (DesktopEntry *entry)
   g_free (entry->name);
   entry->name = NULL;
 
+  g_free (entry->generic_name);
+  entry->generic_name = NULL;
+
   g_free (entry->comment);
   entry->comment = NULL;
 
@@ -356,6 +377,7 @@ desktop_entry_reload (DesktopEntry *entry)
   g_free (entry->exec);
   entry->exec = NULL;
 
+  entry->terminal = 0;
   entry->flags = 0;
 
   return desktop_entry_load (entry);
@@ -383,15 +405,17 @@ desktop_entry_copy (DesktopEntry *entry)
 
   retval = g_new0 (DesktopEntry, 1);
 
-  retval->refcount = 1;
-  retval->type     = entry->type;
-  retval->basename = g_strdup (entry->basename);
-  retval->path     = g_strdup (entry->path);
-  retval->name     = g_strdup (entry->name);
-  retval->comment  = g_strdup (entry->comment);
-  retval->icon     = g_strdup (entry->icon);
-  retval->exec     = g_strdup (entry->exec);
-  retval->flags    = entry->flags;
+  retval->refcount     = 1;
+  retval->type         = entry->type;
+  retval->basename     = g_strdup (entry->basename);
+  retval->path         = g_strdup (entry->path);
+  retval->name         = g_strdup (entry->name);
+  retval->generic_name = g_strdup (entry->generic_name);
+  retval->comment      = g_strdup (entry->comment);
+  retval->icon         = g_strdup (entry->icon);
+  retval->exec         = g_strdup (entry->exec);
+  retval->terminal     = entry->terminal;
+  retval->flags        = entry->flags;
 
   i = 0;
   if (entry->categories != NULL)
@@ -426,6 +450,9 @@ desktop_entry_unref (DesktopEntry *entry)
       g_free (entry->name);
       entry->name = NULL;
 
+      g_free (entry->generic_name);
+      entry->generic_name = NULL;
+
       g_free (entry->comment);
       entry->comment = NULL;
 
@@ -469,6 +496,12 @@ desktop_entry_get_name (DesktopEntry *entry)
   return entry->name;
 }
 
+const char *
+desktop_entry_get_generic_name (DesktopEntry *entry)
+{
+  return entry->generic_name;
+}
+
 const char *
 desktop_entry_get_comment (DesktopEntry *entry)
 {
@@ -487,6 +520,12 @@ desktop_entry_get_exec (DesktopEntry *entry)
   return entry->exec;
 }
 
+gboolean
+desktop_entry_get_launch_in_terminal (DesktopEntry *entry)
+{
+  return entry->terminal;
+}
+
 gboolean
 desktop_entry_get_hidden (DesktopEntry *entry)
 {
